﻿Type=Class
Version=4.2
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'Class module tfGetterClass
'The getter class can be used instead the listener class. 
'The getter is triggered via a timer from the main class. The advantage is to control the getting of measurements at once in regular intervals instead of the heavy load using listeners as these are updated every time a value has changed.
Private Sub Class_Globals
	'Javaobject used for the bricklet listeners
	Private joInline As JavaObject = Me
	'TinkerForge Host and Port
	Private host As String					'ignore
	Private port As Int						'ignore
	'Master Brick UID
	Private masterbrickuid As String		'ignore
	'Bricklet UID
	Private ambientlightuid As String		'ignore
	Private uvlightuid As String			'ignore
	Private distanceuid As String			'ignore
	Private moistureuid As String			'ignore
	'Not used = for tests
	Private temperatureuid As String		'ignore
	Private humidityuid As String			'ignore
	Private barometeruid As String			'ignore
	Private interval As Long = 5000			'ignore
End Sub

'Initializes the object. 
Public Sub Initialize
	Log("Starting TinkerForge Bricklet Listeners.")
	'Get the settings and assign to the class global vars
	host 			= Main.SettingsMap.Get("host")
	port 			= Main.SettingsMap.Get("port")
	interval 		= Main.SettingsMap.Get("interval")
	'Brick
	masterbrickuid	= Main.SettingsMap.get("masterbrickuid")
	'Bricklets
	ambientlightuid = Main.SettingsMap.Get("ambientlightuid")
	uvlightuid 		= Main.SettingsMap.Get("uvlightuid")
	distanceuid 	= Main.SettingsMap.Get("distanceuid")
	moistureuid 	= Main.SettingsMap.Get("moistureuid")
	'Not used = for tests
	temperatureuid	= Main.SettingsMap.Get("temperatureuid")
	humidityuid 	= Main.SettingsMap.Get("humidityuid")
	barometeruid 	= Main.SettingsMap.Get("barometeruid")
End Sub

Public Sub SetValues
	'Start the listener. An exception is thrown if not possible to connect
	Try
		joInline.RunMethod("TinkerForgeGetter", Null)	
		Log($"${DateTime.Time(DateTime.now)} TinkerForge Getter successful executed."$)
	Catch
		Log($"${DateTime.Time(DateTime.now)} TinkerForge Getter Error:${LastException.Message}."$)
	End Try
End Sub

'Update the gartenhaus information device with last update time
Private Sub SetTimeStamp	'ignore
	Dim UpdateMsg As String = $"${DateTime.Time(DateTime.now)}"$
	Main.mqtt_Publish($"{"idx":${Main.SettingsMap.Get("domoticzinfomessageidx")},"nvalue":0,"svalue":"${UpdateMsg}"}"$)
End Sub

'Define the listener result events
'Ensure the domoticz idx topics are in lowercase
Private Sub MasterBrick(chiptemperature As Short)	'ignore
	chiptemperature = chiptemperature / 10
	'Publish the chip temperature - NOT USED as handled by the alert below
	'Dim value As String = $"Chip Temperature ${chiptemperature}"$
	'Main.mqtt_Publish($"{"idx":${Main.SettingsMap.Get("domoticzmasterbrickstatusidx")},"nvalue":0,"svalue":"${value}"}"$)

	'Publish the Master Brick Chip Temperature Alert
	'Build the Alert sensor string according Domoticz API documentation /json.htm?type=command&param=udevice&idx=IDX&nvalue=LEVEL&svalue=TEXT
	'	IDX = id of your device (This number can be found in the devices TAB in the column "IDX")
   	'	Level = (0=gray, 1=green, 2=yellow, 3=orange, 4=red)
   	'	TEXT = Text to display
	'Set the level
	'Set the inital level and status = all fine
	Dim Level As Int = 1
	Dim Status As String = $"${NumberFormat(chiptemperature,0,1)} °C"$
	'Check the CPU Temperature. The threshold is set in settings.map.
	Dim chiptemperaturethreshold As Short = Main.SettingsMap.Get("domoticzmasterbrickchiptempthreshold")
	If chiptemperature > chiptemperaturethreshold Then 
		Level = 2
		Status = $"${NumberFormat(chiptemperature,0,1)} above ${Main.SettingsMap.Get("domoticzmasterbrickchiptempthreshold")} °C"$
	End If
	Main.mqtt_Publish($"{"idx":${Main.SettingsMap.Get("domoticzmasterbrickalertidx")},"nvalue":${Level},"svalue":"${Status}"}"$)
End Sub

Private Sub Illuminance(value As Int)	'ignore
	Main.mqtt_Publish($"{"idx":${Main.SettingsMap.Get("domoticzluxidx")},"nvalue":0,"svalue":"${value / 10.0}"}"$)
End Sub

Private Sub UVLight(value As Long)	'ignore
	'Log("UV Light=" & value & ", " & (value /250.0/250.0) )
	value = value / 250.0 / 250.0
	Main.mqtt_publish($"{"idx":${Main.SettingsMap.Get("domoticzuvidx")},"nvalue":0,"svalue":"${NumberFormat(value,1,2)};0"}"$)
End Sub

Private Sub Distance(value As Int)	'ignore
	Main.mqtt_publish($"{"idx":${Main.SettingsMap.Get("domoticzdistanceidx")},"nvalue":0,"svalue":"${value / 10}"}"$)
End Sub

Private Sub Moisture(value As Int)	'ignore
	'value ranges from 0 = dry -> 4095 = wet:
	'Convert to domoticz moisture<10 = saturated; moisture<20 = adequately wet; moisture<60 = irrigation advise; moisture<100 = irrigation; >100 = dangerously dry
	'Domoticz cb = 100% - (tinkerforge value / 40.95)
	value = 100 - (value / 40.95)
	'Define the Payload string which is the same as humidity
	Dim MoistureStat As Int
	If (value < 10) Then MoistureStat = 0
	If (value >= 10) And (value <20) Then MoistureStat = 1
	If (value >= 20) And (value <60) Then MoistureStat = 2
	If (value >= 60) And (value <100) Then MoistureStat = 3
	If (value >= 100) Then MoistureStat = 4
	Main.mqtt_publish($"{"idx":${Main.SettingsMap.Get("domoticzsoilmoistureidx")},"nvalue":${value},"svalue":"${MoistureStat}"}"$)
End Sub

#Region DEVICESNOTUSED
'
'NOT USED
'
Private Sub Temperature(value As Int)	'ignore
	Log($"${DateTime.time(DateTime.now)}: Temperature= ${value / 100.0}"$)
End Sub

Private Sub Humidity(value As Int)	'ignore
	Log($"${DateTime.time(DateTime.now)}: Humidity= ${value / 10.0}"$)
End Sub

Private Sub AirPressure(value As Int)	'ignore
	Log($"${DateTime.time(DateTime.now)}: AirPressure= ${value / 1000.0}"$)
End Sub
#End Region

'Define the listeners. Be exact in case - lookup the API reference with the examples
#If JAVA
import com.tinkerforge.IPConnection;

//Listener for the bricks and bricklets
//Define the imports

import com.tinkerforge.BrickMaster;
//ref: www.tinkerforge.com/en/doc/Software/Bricks/Master_Brick_Java.html#master-brick-java-examples

import com.tinkerforge.BrickletAmbientLight;
//ref: www.tinkerforge.com/en/doc/Software/Bricklets/AmbientLight_Bricklet_Java.html#ambient-light-bricklet-java-examples

import com.tinkerforge.BrickletUVLight;
//ref: www.tinkerforge.com/en/doc/Software/Bricklets/UVLight_Bricklet_Java.html#uv-light-bricklet-java-examples

import com.tinkerforge.BrickletDistanceIR;
//ref: www.tinkerforge.com/en/doc/Software/Bricklets/DistanceIR_Bricklet_Java.html#distance-ir-bricklet-java-examples

import com.tinkerforge.BrickletMoisture;
//ref: www.tinkerforge.com/en/doc/Software/Bricklets/Moisture_Bricklet_Java.html#moisture-bricklet-java-examples

import com.tinkerforge.BrickletTemperature;
//Ref: www.tinkerforge.com/en/doc/Software/Bricklets/Temperature_Bricklet_Java.html#temperature-bricklet-java-examples

import com.tinkerforge.BrickletHumidity;
//Ref: http://www.tinkerforge.com/en/doc/Software/Bricklets/Humidity_Bricklet_Java.html#humidity-bricklet-java-examples

import com.tinkerforge.BrickletBarometer;
//Ref: www.tinkerforge.com/en/doc/Software/Bricklets/Barometer_Bricklet_Java.html#barometer-bricklet-java-examples

//ADD MORE

//Define the TinkerForge Getter
public void TinkerForgeGetter() throws Exception {
	IPConnection ipcon = new IPConnection(); 

	//Define the bricks
	BrickMaster master = new BrickMaster(_masterbrickuid, ipcon);

	//Define the brickets
	BrickletAmbientLight al = new BrickletAmbientLight(_ambientlightuid, ipcon);
	BrickletUVLight uvl = new BrickletUVLight(_uvlightuid, ipcon);
	BrickletDistanceIR dir = new BrickletDistanceIR(_distanceuid, ipcon);
	BrickletMoisture m = new BrickletMoisture(_moistureuid, ipcon);

	//NOT USED
	/*
	BrickletTemperature t = new BrickletTemperature(_temperatureuid, ipcon); 
  	BrickletHumidity h = new BrickletHumidity(_humidityuid, ipcon); 
  	BrickletBarometer b = new BrickletBarometer(_barometeruid, ipcon); 
	*/
	//ADD MORE
	
	//Make the ip connection
	ipcon.connect(_host, _port); 

	//MASTERBRICK
	// Get USB voltage - not used as not working for hw version 2.1
	int usbVoltage = master.getUSBVoltage();
		//System.out.println("USB Voltage: " + usbVoltage + " V");
	// Get current stack voltage (unit is mV) - has only a value if step down power supply
	int stackVoltage = master.getStackVoltage();
		//System.out.println("Stack Voltage: " + stackVoltage/1000.0 + " V");
	// Get current stack current (unit is mA) - has only a value if step down power supply
	int stackCurrent = master.getStackCurrent();
		//System.out.println("Stack Current: " + stackCurrent/1000.0 + " A");
	// Get the chip temperature. Only use this value as an indicator. - this value is used
	short chipTemperature = master.getChipTemperature();
		//System.out.println("Chip Temperature: " + chipTemperature + " C");
	//Raise the BA evenmt to update the values
	ba.raiseEvent(this, "masterbrick", chipTemperature);
	
  	//AMBIENTLIGHT
	// Get current illuminance (unit is Lux/10)
	ba.raiseEvent(this, "illuminance", al.getIlluminance());
  	//UVLIGHT
	// Get current UV light (unit is µW/cm²)
	ba.raiseEvent(this, "uvlight", uvl.getUVLight());
	//DISTANCE
	// Get current distance (unit is mm)
	ba.raiseEvent(this, "distance", dir.getDistance());
	//MOISTURE
	// Get current moisture value
	// The value has a range of 0 to 4095. A small value corresponds to little moisture, a big value corresponds to much moisture.
	ba.raiseEvent(this, "moisture", m.getMoistureValue());

	//ADD MORE

	// Set the timestamp
	ba.raiseEvent(this, "settimestamp", null);

	ipcon.disconnect();
}

#End If
