﻿Type=Class
Version=4.2
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'Class module tfListenerClass
Sub Class_Globals
	'Javaobject used for the bricklet listeners
	Private joInline As JavaObject = Me
	'TinkerForge Host and Port
	Private host As String					'ignore
	Private port As Int						'ignore
	'Master Brick UID
	Private masterbrickuid As String		'ignore
	'Bricklet UID
	Private ambientlightuid As String		'ignore
	Private uvlightuid As String			'ignore
	Private distanceuid As String			'ignore
	Private moistureuid As String			'ignore
	'Not used = for tests
	Private temperatureuid As String		'ignore
	Private humidityuid As String			'ignore
	Private barometeruid As String			'ignore
	Private interval As Long = 5000			'ignore
End Sub

'Initializes the object. 
Public Sub Initialize
	Log("Starting TinkerForge Bricklet Listeners.")
	'Get the settings and assign to the class global vars
	host 			= Main.SettingsMap.Get("host")
	port 			= Main.SettingsMap.Get("port")
	interval 		= Main.SettingsMap.Get("interval")
	'Brick
	masterbrickuid	= Main.SettingsMap.get("masterbrickuid")
	'Bricklets
	ambientlightuid = Main.SettingsMap.Get("ambientlightuid")
	uvlightuid 		= Main.SettingsMap.Get("uvlightuid")
	distanceuid 	= Main.SettingsMap.Get("distanceuid")
	moistureuid 	= Main.SettingsMap.Get("moistureuid")
	'Not used = for tests
	temperatureuid	= Main.SettingsMap.Get("temperatureuid")
	humidityuid 	= Main.SettingsMap.Get("humidityuid")
	barometeruid 	= Main.SettingsMap.Get("barometeruid")
	'Start the listener. An exception is thrown if not possible to connect
	Try
		joInline.RunMethod("TinkerForgeListener", Null)	
		Log($"TinkerForge Listener started (Interval ms=${interval})."$)
	Catch
		Log($"TinkerForge Listener Error:${LastException.Message}."$)
	End Try
End Sub

'Define the listener result events
'Ensure the domoticz idx topics are in lowercase
Private Sub MasterBrick(voltage As Int, current As Int, chiptemperature As Short)	'ignore
	Dim value As String = $"V=${voltage}, A=${current}, T=${chiptemperature / 10}"$
	Main.mqtt_Publish($"{"idx":${Main.SettingsMap.Get("domoticzmasterbrickstatus")},"nvalue":0,"svalue":"${value}"}"$)
End Sub

Private Sub Illuminance(value As Int)	'ignore
	Main.mqtt_Publish($"{"idx":${Main.SettingsMap.Get("domoticzluxidx")},"nvalue":0,"svalue":"${value / 10.0}"}"$)
End Sub

Private Sub UVLight(value As Long)	'ignore
	Main.mqtt_publish($"{"idx":${Main.SettingsMap.Get("domoticzuvidx")},"nvalue":0,"svalue":"${NumberFormat(value / 250,1,2)};0"}"$)
End Sub

Private Sub Distance(value As Int)	'ignore
	Main.mqtt_publish($"{"idx":${Main.SettingsMap.Get("domoticzdistanceidx")},"nvalue":0,"svalue":"${value / 10}"}"$)
End Sub

Private Sub Moisture(value As Int)	'ignore
	'value ranges from 0 = dry -> 4095 = wet:
	'Convert to domoticz moisture<10 = saturated; moisture<20 = adequately wet; moisture<60 = irrigation advise; moisture<100 = irrigation; >100 = dangerously dry
	'Domoticz cb = 100% - (tinkerforge value / 40.95)
	value = 100 - (value / 40.95)
	'Define the Payload string which is the same as humidity
	Dim MoistureStat As Int
	If (value < 10) Then MoistureStat = 0
	If (value >= 10) And (value <20) Then MoistureStat = 1
	If (value >= 20) And (value <60) Then MoistureStat = 2
	If (value >= 60) And (value <100) Then MoistureStat = 3
	If (value >= 100) Then MoistureStat = 4
	Main.mqtt_publish($"{"idx":${Main.SettingsMap.Get("domoticzsoilmoistureidx")},"nvalue":${value},"svalue":"${MoistureStat}"}"$)
End Sub

Private Sub Temperature(value As Int)	'ignore
	Log($"${DateTime.time(DateTime.now)}: Temperature= ${value / 100.0}"$)
End Sub

Private Sub Humidity(value As Int)	'ignore
	Log($"${DateTime.time(DateTime.now)}: Humidity= ${value / 10.0}"$)
End Sub

Private Sub AirPressure(value As Int)	'ignore
	Log($"${DateTime.time(DateTime.now)}: AirPressure= ${value / 1000.0}"$)
End Sub

'Define the listeners. Be exact in case - lookup the API reference with the examples
#If JAVA
import com.tinkerforge.IPConnection;

//Listener for the bricks and bricklets
//Define the imports

import com.tinkerforge.BrickMaster;
//ref: www.tinkerforge.com/en/doc/Software/Bricks/Master_Brick_Java.html#master-brick-java-examples

import com.tinkerforge.BrickletAmbientLight;
//ref: www.tinkerforge.com/en/doc/Software/Bricklets/AmbientLight_Bricklet_Java.html#ambient-light-bricklet-java-examples

import com.tinkerforge.BrickletUVLight;
//ref: www.tinkerforge.com/en/doc/Software/Bricklets/UVLight_Bricklet_Java.html#uv-light-bricklet-java-examples

import com.tinkerforge.BrickletDistanceIR;
//ref: www.tinkerforge.com/en/doc/Software/Bricklets/DistanceIR_Bricklet_Java.html#distance-ir-bricklet-java-examples

import com.tinkerforge.BrickletMoisture;
//ref: www.tinkerforge.com/en/doc/Software/Bricklets/Moisture_Bricklet_Java.html#moisture-bricklet-java-examples

import com.tinkerforge.BrickletTemperature;
//Ref: www.tinkerforge.com/en/doc/Software/Bricklets/Temperature_Bricklet_Java.html#temperature-bricklet-java-examples

import com.tinkerforge.BrickletHumidity;
//Ref: http://www.tinkerforge.com/en/doc/Software/Bricklets/Humidity_Bricklet_Java.html#humidity-bricklet-java-examples

import com.tinkerforge.BrickletBarometer;
//Ref: www.tinkerforge.com/en/doc/Software/Bricklets/Barometer_Bricklet_Java.html#barometer-bricklet-java-examples

//ADD MORE

//MASTERBRICK
//Define the MasterBrick Info
public void MasterBrickInfo() throws Exception {
	IPConnection ipcon = new IPConnection(); 
	BrickMaster master = new BrickMaster(_masterbrickuid, ipcon);
	ipcon.connect(_host, _port); 
	// Get current stack voltage (unit is mV)
		int stackVoltage = master.getStackVoltage();
			//System.out.println("Stack Voltage: " + stackVoltage/1000.0 + " V");
		// Get current stack current (unit is mA)
		int stackCurrent = master.getStackCurrent();
			//System.out.println("Stack Current: " + stackCurrent/1000.0 + " A");
		// Get the chip temperature. Only use this value as an indicator.
		short chipTemperature = master.getChipTemperature();
			//System.out.println("Chip Temperature: " + chipTemperature + " C");
		//Raise the BA evemmt to update the values
		ba.raiseEvent(this, "masterbrick", stackVoltage, stackCurrent, chipTemperature);

};

//Define the TinkerForge Listeners
public void TinkerForgeListener() throws Exception {
	IPConnection ipcon = new IPConnection(); 

	//Define the bricks
	BrickMaster master = new BrickMaster(_masterbrickuid, ipcon);

	//Define the brickets
	BrickletAmbientLight al = new BrickletAmbientLight(_ambientlightuid, ipcon);
	BrickletUVLight uvl = new BrickletUVLight(_uvlightuid, ipcon);
	BrickletDistanceIR dir = new BrickletDistanceIR(_distanceuid, ipcon);
	BrickletMoisture m = new BrickletMoisture(_moistureuid, ipcon);

	//NOT USED
	/*
	BrickletTemperature t = new BrickletTemperature(_temperatureuid, ipcon); 
  	BrickletHumidity h = new BrickletHumidity(_humidityuid, ipcon); 
  	BrickletBarometer b = new BrickletBarometer(_barometeruid, ipcon); 
	*/
	//ADD MORE
	
	//Make the ip connection
	ipcon.connect(_host, _port); 

	//MASTERBRICK
	MasterBrickInfo();
	
  	//AMBIENTLIGHT
	// Get current illuminance (unit is Lux/10)
	ba.raiseEvent(this, "illuminance", al.getIlluminance());
	al.addIlluminanceListener(new BrickletAmbientLight.IlluminanceListener() {
		public void illuminance(int illuminance) {
			//System.out.println("Illuminance: " + illuminance/10.0 + " Lux");
			ba.raiseEvent(this, "illuminance", illuminance);
		}
	});
    al.setIlluminanceCallbackPeriod(_interval);

  	//UVLIGHT
	// Get current UV light (unit is µW/cm²)
	ba.raiseEvent(this, "uvlight", uvl.getUVLight());
	uvl.addUVLightListener(new BrickletUVLight.UVLightListener() {
		public void uvLight(long uvLight) {
			//System.out.println("UV Light: " + uvLight + " µW/cm²");
			ba.raiseEvent(this, "uvlight", uvLight);
		}
	});
	uvl.setUVLightCallbackPeriod(_interval);

	//DISTANCE
	// Get current distance (unit is mm)
	ba.raiseEvent(this, "distance", dir.getDistance());
	dir.addDistanceListener(new BrickletDistanceIR.DistanceListener() {
		public void distance(int distance) {
			//System.out.println("Distance: " + distance/10.0 + " cm");
			ba.raiseEvent(this, "distance", distance);
		}
	});
	dir.setDistanceCallbackPeriod(_interval);	

	//MOISTURE
	// Get current moisture value
	// The value has a range of 0 to 4095. A small value corresponds to little moisture, a big value corresponds to much moisture.
	ba.raiseEvent(this, "moisture", m.getMoistureValue());
	m.addMoistureListener(new BrickletMoisture.MoistureListener() {
		public void moisture(int moisture) {
			//System.out.println("Moisture Value: " + moisture);
			ba.raiseEvent(this, "moisture", moisture);
		}
	});
	m.setMoistureCallbackPeriod(_interval);

	//ADD MORE

	/*
  	//TEMPERATURE
	t.addTemperatureListener(new BrickletTemperature.TemperatureListener() {
		public void temperature(short temperature) {
        	//System.out.println("Temperature: " + temperature/100.0 + " °C");
			ba.raiseEvent(this, "temperature", temperature);
		}
 	});
	t.setTemperatureCallbackPeriod(_interval);

	//HUMIDITY
	h.addHumidityListener(new BrickletHumidity.HumidityListener() {
		public void humidity(int humidity) {
        	//System.out.println("Humidity: " + humidity/10.0 + " %RH");
			ba.raiseEvent(this, "humidity", humidity);
		}
 	});
    h.setHumidityCallbackPeriod(_interval);

	//AIRPRESSURE
 	b.addAirPressureListener(new BrickletBarometer.AirPressureListener() {
		public void airPressure(int airPressure) {
			//System.out.println("Air Pressure: " + airPressure/1000.0 + " mbar");
			ba.raiseEvent(this, "airpressure", airPressure);
		}
   	});
 	b.setAirPressureCallbackPeriod(_interval);
	*/

	//ipcon.disconnect();
}

#End If
